@@ -162,7 +162,8 @@ brasero_src_image_error (BraseroSrcImage *self,
 		return;
 
 	brasero_utils_message_dialog (toplevel,
-				      _("Please select another image."),
+				      /* Translators: this is a disc image, not a picture */
+				      C_("disc", "Please select another image."),
 				      error->message,
 				      GTK_MESSAGE_ERROR);
 }
@@ -456,7 +457,7 @@ brasero_src_image_clicked (GtkButton *button)
 
 	filter = gtk_file_filter_new ();
 	/* Translators: this a disc image here */
-	gtk_file_filter_set_name (filter, C_("disc", "Image files only"));
+	gtk_file_filter_set_name (filter, C_("disc", "Image files"));
 	gtk_file_filter_add_mime_type (filter, mimes [0]);
 	gtk_file_filter_add_mime_type (filter, mimes [1]);
 	gtk_file_filter_add_mime_type (filter, mimes [2]);
