@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #include <string.h>
@@ -31,34 +33,10 @@
 #include <glib.h>
 #include <glib/gi18n-lib.h>
 
-#include <gtk/gtkuimanager.h>
-#include <gtk/gtktoggleaction.h>
-#include <gtk/gtkradioaction.h>
-#include <gtk/gtkaction.h>
-
-#include <gtk/gtkstock.h>
-
-#include <gtk/gtkbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkvpaned.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtkhseparator.h>
-#include <gtk/gtkalignment.h>
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtkcellrenderer.h>
-#include <gtk/gtkcellrendererpixbuf.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkcelllayout.h>
+#include <gtk/gtk.h>
 
 #include <gconf/gconf-client.h>
 
-#include "burn-debug.h"
 #include "brasero-layout.h"
 #include "brasero-preview.h"
 #include "brasero-project.h"
@@ -110,6 +88,7 @@ struct BraseroLayoutPrivate {
 	gint radio_notify;
 	gint preview_notify;
 	gint layout_notify;
+	gint sidepane_notify;
 
 	GtkWidget *notebook;
 	GtkWidget *main_box;
@@ -119,13 +98,13 @@ struct BraseroLayoutPrivate {
 static GObjectClass *parent_class = NULL;
 
 #define BRASERO_LAYOUT_PREVIEW_ID	"Viewer"
-#define BRASERO_LAYOUT_PREVIEW_NAME	N_("Preview")
 #define BRASERO_LAYOUT_PREVIEW_MENU	N_("P_review")
+/* Translators: this is an image, a picture, not a "Disc Image" */
 #define BRASERO_LAYOUT_PREVIEW_TOOLTIP	N_("Display video, audio and image preview")
 #define BRASERO_LAYOUT_PREVIEW_ICON	GTK_STOCK_FILE
 
 #define BRASERO_LAYOUT_NONE_ID		"EmptyView"
-#define BRASERO_LAYOUT_NONE_MENU	N_("_Show side panel")
+#define BRASERO_LAYOUT_NONE_MENU	N_("_Show Side Panel")
 #define BRASERO_LAYOUT_NONE_TOOLTIP	N_("Show a side pane along the project")
 #define BRASERO_LAYOUT_NONE_ICON	NULL
 
@@ -138,17 +117,30 @@ const GtkToggleActionEntry entries [] = {
 	  "F7", N_(BRASERO_LAYOUT_NONE_TOOLTIP), G_CALLBACK (brasero_layout_empty_toggled_cb), 1 }
 };
 
-const gchar description [] = "<ui>"
-				"<menubar name='menubar' >"
-				"<menu action='ViewMenu'>"
-				"<placeholder name='ViewPlaceholder'>"
+/** see #547687 **/
+const GtkRadioActionEntry radio_entries [] = {
+	{ "HView", NULL, N_("_Horizontal Layout"),
+	  NULL, N_("Set an horizontal layout"), 1 },
+
+	{ "VView", NULL, N_("_Vertical Layout"),
+	  NULL, N_("Set a vertical layout"), 0 },
+};
+
+const gchar description [] =
+	"<ui>"
+	"<menubar name='menubar' >"
+		"<menu action='ViewMenu'>"
+			"<placeholder name='ViewPlaceholder'>"
 				"<menuitem action='"BRASERO_LAYOUT_NONE_ID"'/>"
 				"<menuitem action='"BRASERO_LAYOUT_PREVIEW_ID"'/>"
 				"<separator/>"
-				"</placeholder>"
-				"</menu>"
-				"</menubar>"
-			     "</ui>";
+				"<menuitem action='VView'/>"
+				"<menuitem action='HView'/>"
+				"<separator/>"
+			"</placeholder>"
+		"</menu>"
+	"</menubar>"
+	"</ui>";
 
 /* GCONF keys */
 #define BRASERO_KEY_DISPLAY_LAYOUT	"/apps/brasero/display/layout"
@@ -156,9 +148,18 @@ const gchar description [] = "<ui>"
 
 
 #define BRASERO_KEY_DISPLAY_DIR		"/apps/brasero/display/"
-#define BRASERO_KEY_SHOW_PREVIEW	BRASERO_KEY_DISPLAY_DIR "preview"
+#define BRASERO_KEY_SHOW_SIDEPANE	BRASERO_KEY_DISPLAY_DIR "sidepane"
 #define BRASERO_KEY_LAYOUT_AUDIO	BRASERO_KEY_DISPLAY_DIR "audio_pane"
 #define BRASERO_KEY_LAYOUT_DATA		BRASERO_KEY_DISPLAY_DIR "data_pane"
+#define BRASERO_KEY_LAYOUT_VIDEO	BRASERO_KEY_DISPLAY_DIR "video_pane"
+
+/* signals */
+typedef enum {
+	SIDEPANE_SIGNAL,
+	LAST_SIGNAL
+} BraseroLayoutSignalType;
+
+static guint brasero_layout_signals [LAST_SIGNAL] = { 0 };
 
 static void
 brasero_layout_pack_preview (BraseroLayout *layout)
@@ -369,10 +370,14 @@ brasero_layout_preview_changed_cb (GConfClient *client,
 					      BRASERO_LAYOUT_PREVIEW_ID);
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), active);
 
- 	if (active)
+	if (active) {
+		brasero_preview_set_enabled (BRASERO_PREVIEW (layout->priv->preview_pane), TRUE);
 		gtk_widget_show (layout->priv->preview_pane);
-	else
+	}
+	else {
+		brasero_preview_set_enabled (BRASERO_PREVIEW (layout->priv->preview_pane), FALSE);
 		gtk_widget_hide (layout->priv->preview_pane);
+	}
 }
 
 void
@@ -418,10 +423,14 @@ brasero_layout_add_preview (BraseroLayout *layout,
 	action = gtk_action_group_get_action (layout->priv->action_group, BRASERO_LAYOUT_PREVIEW_ID);
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), active);
 
-	if (active)
+	if (active) {
+		brasero_preview_set_enabled (BRASERO_PREVIEW (layout->priv->preview_pane), TRUE);
 		gtk_widget_show (layout->priv->preview_pane);
-	else
+	}
+	else {
+		brasero_preview_set_enabled (BRASERO_PREVIEW (layout->priv->preview_pane), FALSE);
 		gtk_widget_hide (layout->priv->preview_pane);
+	}
 
 	if (!layout->priv->preview_notify)
 		layout->priv->preview_notify = gconf_client_notify_add (layout->priv->client,
@@ -430,6 +439,7 @@ brasero_layout_add_preview (BraseroLayout *layout,
 									layout,
 									NULL,
 									&error);
+
 	if (error) {
 		g_warning ("Could set notify for GConf key %s.\n", error->message);
 		g_error_free (error);
@@ -487,16 +497,22 @@ brasero_layout_set_side_pane_visible (BraseroLayout *layout,
 		/* Now tell the project which source it gets URIs from */
 		source = brasero_layout_item_get_object (layout->priv->active_item);
 		if (!BRASERO_IS_URI_CONTAINER (source)) {
-			BRASERO_BURN_LOG ("Item is not an URI container");
 			brasero_project_set_source (BRASERO_PROJECT (layout->priv->project),
 						    NULL);
 		}
-		else
+		else {
 			brasero_project_set_source (BRASERO_PROJECT (layout->priv->project),
 						    BRASERO_URI_CONTAINER (source));
+			brasero_uri_container_uri_selected (BRASERO_URI_CONTAINER (source));
+		}
 
 		gtk_widget_show (layout->priv->main_box->parent);
 	}
+
+	g_signal_emit (layout,
+		       brasero_layout_signals [SIDEPANE_SIGNAL],
+		       0,
+		       visible);
 }
 
 static void
@@ -518,13 +534,42 @@ brasero_layout_item_set_active (BraseroLayout *layout,
 				    -1);
 
 		if (tree_item == item) {
+			BraseroLayoutObject *object;
+
 			gtk_combo_box_set_active_iter (GTK_COMBO_BOX (layout->priv->combo), &iter);
 			gtk_widget_show (item->widget);
 			layout->priv->active_item = item;
+
+			/* tell the object what context we are in */
+			object = brasero_layout_item_get_object (item);
+			brasero_layout_object_set_context (object, layout->priv->ctx_type);
 			return;
 		}
 	} while (gtk_tree_model_iter_next (model, &iter));
+}
 
+static void
+brasero_layout_show_sidepane_changed_cb (GConfClient *client,
+					 guint cxn,
+					 GConfEntry *entry,
+					 gpointer data)
+{
+	BraseroLayout *layout;
+	GtkAction *action;
+	GConfValue *value;
+	gboolean show;
+
+	value = gconf_entry_get_value (entry);
+	if (value->type != GCONF_VALUE_BOOL)
+		return;
+
+	show = gconf_value_get_bool (value);
+
+	layout = BRASERO_LAYOUT (data);
+
+	action = gtk_action_group_get_action (layout->priv->action_group, BRASERO_LAYOUT_NONE_ID);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), show);
+	brasero_layout_set_side_pane_visible (layout, show);
 }
 
 static void
@@ -552,6 +597,10 @@ brasero_layout_displayed_item_changed_cb (GConfClient *client,
 	&&  layout->priv->ctx_type == BRASERO_LAYOUT_DATA)
 		return;
 
+	if (!strcmp (entry->key, BRASERO_KEY_LAYOUT_VIDEO)
+	&&  layout->priv->ctx_type == BRASERO_LAYOUT_VIDEO)
+		return;
+
 	value = gconf_entry_get_value (entry);
 	if (value->type != GCONF_VALUE_STRING)
 		return;
@@ -635,6 +684,25 @@ brasero_layout_save (BraseroLayout *layout,
 								      NULL,
 								      &error);
 	}
+	else if (layout->priv->ctx_type == BRASERO_LAYOUT_VIDEO) {
+		gconf_client_set_string (layout->priv->client,
+					 BRASERO_KEY_LAYOUT_VIDEO,
+					 id,
+					 &error);
+
+		if (error) {
+			g_warning ("Can't set GConf key %s. \n", error->message);
+			g_error_free (error);
+			error = NULL;
+		}
+
+		layout->priv->radio_notify = gconf_client_notify_add (layout->priv->client,
+								      BRASERO_KEY_LAYOUT_VIDEO,
+								      brasero_layout_displayed_item_changed_cb,
+								      layout,
+								      NULL,
+								      &error);
+	}
 
 	if (error) {
 		g_warning ("Can't set GConf notify on key %s. \n", error->message);
@@ -648,7 +716,6 @@ brasero_layout_add_source (BraseroLayout *layout,
 			   GtkWidget *source,
 			   const gchar *id,
 			   const gchar *subtitle,
-			   const gchar *tooltip,
 			   const gchar *icon_name,
 			   BraseroLayoutType types)
 {
@@ -693,6 +760,8 @@ brasero_layout_combo_changed_cb (GtkComboBox *combo,
 	BraseroLayoutObject *source;
 	BraseroLayoutItem *item;
 	GtkTreeModel *model;
+	gboolean is_visible;
+	GtkAction *action;
 	GtkTreeIter iter;
 
 	model = gtk_combo_box_get_model (combo);
@@ -703,21 +772,33 @@ brasero_layout_combo_changed_cb (GtkComboBox *combo,
 			    ITEM_COL, &item,
 			    -1);
 
+	/* Make sure there is a displayed sidepane before setting the source for
+	 * project. It can happen that when we're changing of project type this
+	 * is called. */
+	action = gtk_action_group_get_action (layout->priv->action_group, BRASERO_LAYOUT_NONE_ID);
+	is_visible = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+
 	if (layout->priv->active_item)
 		gtk_widget_hide (layout->priv->active_item->widget);
 
 	layout->priv->active_item = item;
 	gtk_widget_show (item->widget);
 
+	/* tell the object what context we are in */
 	source = brasero_layout_item_get_object (item);
+	brasero_layout_object_set_context (source, layout->priv->ctx_type);
+
 	if (!BRASERO_IS_URI_CONTAINER (source)) {
-		BRASERO_BURN_LOG ("Item is not an URI container");
-		brasero_project_set_source (BRASERO_PROJECT (layout->priv->project),
-					    NULL);
+		brasero_project_set_source (BRASERO_PROJECT (layout->priv->project), NULL);
 	}
-	else
+	else if (!is_visible) {
+		brasero_project_set_source (BRASERO_PROJECT (layout->priv->project), NULL);
+	}
+	else {
+		brasero_uri_container_uri_selected (BRASERO_URI_CONTAINER (source));
 		brasero_project_set_source (BRASERO_PROJECT (layout->priv->project),
 					    BRASERO_URI_CONTAINER (source));
+	}
 
 	brasero_layout_save (layout, item->id);
 }
@@ -758,12 +839,14 @@ brasero_layout_item_set_visible (BraseroLayout *layout,
 }
 
 void
-brasero_layout_load (BraseroLayout *layout, BraseroLayoutType type)
+brasero_layout_load (BraseroLayout *layout,
+		     BraseroLayoutType type)
 {
 	gchar *layout_id = NULL;
 	GError *error = NULL;
 	GtkTreeModel *model;
 	GtkAction *action;
+	gboolean sidepane;
 	GtkTreeIter iter;
 
 	/* remove GCONF notification if any */
@@ -792,6 +875,10 @@ brasero_layout_load (BraseroLayout *layout, BraseroLayoutType type)
 		layout_id = gconf_client_get_string (layout->priv->client,
 						     BRASERO_KEY_LAYOUT_DATA,
 						     &error);
+	else if (type == BRASERO_LAYOUT_VIDEO)
+		layout_id = gconf_client_get_string (layout->priv->client,
+						     BRASERO_KEY_LAYOUT_VIDEO,
+						     &error);
 
 	if (error) {
 		g_warning ("Can't access GConf key %s. This is probably harmless (first launch of brasero).\n", error->message);
@@ -814,6 +901,13 @@ brasero_layout_load (BraseroLayout *layout, BraseroLayoutType type)
 								      layout,
 								      NULL,
 								      &error);
+	else if (type == BRASERO_LAYOUT_VIDEO)
+		layout->priv->radio_notify = gconf_client_notify_add (layout->priv->client,
+								      BRASERO_KEY_LAYOUT_VIDEO,
+								      brasero_layout_displayed_item_changed_cb,
+								      layout,
+								      NULL,
+								      &error);
 
 	if (error) {
 		g_warning ("Could not set notify for GConf key %s.\n", error->message);
@@ -828,23 +922,18 @@ brasero_layout_load (BraseroLayout *layout, BraseroLayoutType type)
 		gtk_widget_hide (layout->priv->active_item->widget);
 		layout->priv->active_item = NULL;
 	}
+
 	layout->priv->ctx_type = type;
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (layout->priv->combo));
 	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
 	if (gtk_tree_model_get_iter_first (model, &iter)) {
 		do {
-			BraseroLayoutObject *object;
 			BraseroLayoutItem *item = NULL;
 
 			gtk_tree_model_get (model, &iter,
 					    ITEM_COL, &item,
 					    -1);
 
-			/* tell all the object what context we are in */
-			object = brasero_layout_item_get_object (item);
-			if (object)
-				brasero_layout_object_set_context (object, type);
-
 			/* check if that pane should be displayed in such a context */
 			if (!(item->types & type)) {
 				brasero_layout_item_set_visible (layout, item, FALSE);
@@ -874,10 +963,12 @@ brasero_layout_load (BraseroLayout *layout, BraseroLayoutType type)
 	}
 
 	/* hide or show side pane */
-	action = gtk_action_group_get_action (layout->priv->action_group, BRASERO_LAYOUT_NONE_ID);
-	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
-				     (layout_id && strcmp (layout_id, BRASERO_LAYOUT_NONE_ID)));
+	sidepane = gconf_client_get_bool (layout->priv->client,
+					  BRASERO_KEY_SHOW_SIDEPANE,
+					  NULL);
 
+	action = gtk_action_group_get_action (layout->priv->action_group, BRASERO_LAYOUT_NONE_ID);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), sidepane);
 	g_free (layout_id);
 }
 
@@ -893,47 +984,39 @@ brasero_layout_pane_moved_cb (GtkWidget *paned,
 }
 
 static void
-brasero_layout_type_changed_cb (GConfClient *client,
-				guint cxn,
-				GConfEntry *entry,
-				gpointer data)
+brasero_layout_change_type (BraseroLayout *layout,
+			    BraseroLayoutType layout_type)
 {
-	GConfValue *value;
-	GtkWidget *source_pane;
-	GtkWidget *project_pane;
-	BraseroLayoutType layout_type;
-	BraseroLayout *layout = BRASERO_LAYOUT (data);
+	GtkWidget *source_pane = NULL;
+	GtkWidget *project_pane = NULL;
 
-	value = gconf_entry_get_value (entry);
-	if (value->type != GCONF_VALUE_INT)
-		return;
+	if (layout->priv->pane) {
+		g_object_ref (layout->priv->preview_pane);
+		if (layout->priv->layout_type == BRASERO_LAYOUT_BOTTOM)
+			gtk_container_remove (GTK_CONTAINER (layout->priv->project), layout->priv->preview_pane);
+		else
+			gtk_container_remove (GTK_CONTAINER (layout->priv->main_box), layout->priv->preview_pane);
 
-	g_object_ref (layout->priv->preview_pane);
-	if (layout->priv->layout_type == BRASERO_LAYOUT_BOTTOM)
-		gtk_container_remove (GTK_CONTAINER (layout->priv->project), layout->priv->preview_pane);
-	else
-		gtk_container_remove (GTK_CONTAINER (layout->priv->main_box), layout->priv->preview_pane);
+		if (layout->priv->layout_type == BRASERO_LAYOUT_TOP
+		||  layout->priv->layout_type == BRASERO_LAYOUT_LEFT) {
+			project_pane = gtk_paned_get_child1 (GTK_PANED (layout->priv->pane));
+			source_pane = gtk_paned_get_child2 (GTK_PANED (layout->priv->pane));
+		}
+		else {
+			source_pane = gtk_paned_get_child1 (GTK_PANED (layout->priv->pane));
+			project_pane = gtk_paned_get_child2 (GTK_PANED (layout->priv->pane));
+		}
 
-	if (layout->priv->layout_type == BRASERO_LAYOUT_TOP
-	||  layout->priv->layout_type == BRASERO_LAYOUT_LEFT) {
-		project_pane = gtk_paned_get_child1 (GTK_PANED (layout->priv->pane));
-		source_pane = gtk_paned_get_child2 (GTK_PANED (layout->priv->pane));
-	}
-	else {
-		source_pane = gtk_paned_get_child1 (GTK_PANED (layout->priv->pane));
-		project_pane = gtk_paned_get_child2 (GTK_PANED (layout->priv->pane));
+		g_object_ref (source_pane);
+		gtk_container_remove (GTK_CONTAINER (layout->priv->pane), source_pane);
+
+		g_object_ref (project_pane);
+		gtk_container_remove (GTK_CONTAINER (layout->priv->pane), project_pane);
+
+		gtk_widget_destroy (layout->priv->pane);
+		layout->priv->pane = NULL;
 	}
 
-	g_object_ref (source_pane);
-	gtk_container_remove (GTK_CONTAINER (layout->priv->pane), source_pane);
-
-	g_object_ref (project_pane);
-	gtk_container_remove (GTK_CONTAINER (layout->priv->pane), project_pane);
-
-	gtk_widget_destroy (layout->priv->pane);
-	layout->priv->pane = NULL;
-
-	layout_type = gconf_value_get_int (value);
 	if (layout_type > BRASERO_LAYOUT_BOTTOM
 	||  layout_type < BRASERO_LAYOUT_RIGHT)
 		layout_type = BRASERO_LAYOUT_RIGHT;
@@ -956,38 +1039,122 @@ brasero_layout_type_changed_cb (GConfClient *client,
 	gtk_widget_show (layout->priv->pane);
 	gtk_box_pack_end (GTK_BOX (layout), layout->priv->pane, TRUE, TRUE, 0);
 
-	switch (layout_type) {
-		case BRASERO_LAYOUT_TOP:
-		case BRASERO_LAYOUT_LEFT:
-			gtk_paned_pack2 (GTK_PANED (layout->priv->pane), source_pane, FALSE, FALSE);
-			gtk_paned_pack1 (GTK_PANED (layout->priv->pane), project_pane, TRUE, FALSE);
-			break;
-
-		case BRASERO_LAYOUT_BOTTOM:
-		case BRASERO_LAYOUT_RIGHT:
-			gtk_paned_pack2 (GTK_PANED (layout->priv->pane), project_pane, TRUE, FALSE);
-			gtk_paned_pack1 (GTK_PANED (layout->priv->pane), source_pane, FALSE, FALSE);
-			break;
-
-		default:
-			break;
-	}
-
 	g_signal_connect (layout->priv->pane,
 			  "notify::position",
 			  G_CALLBACK (brasero_layout_pane_moved_cb),
 			  layout);
 
 	layout->priv->layout_type = layout_type;
-	g_object_unref (project_pane);
-	g_object_unref (source_pane);
 
-	brasero_layout_pack_preview (layout);
-	g_object_unref (layout->priv->preview_pane);
+	if (source_pane && project_pane) {
+		switch (layout_type) {
+			case BRASERO_LAYOUT_TOP:
+			case BRASERO_LAYOUT_LEFT:
+				gtk_paned_pack2 (GTK_PANED (layout->priv->pane), source_pane, TRUE, TRUE);
+				gtk_paned_pack1 (GTK_PANED (layout->priv->pane), project_pane, TRUE, FALSE);
+				break;
+
+			case BRASERO_LAYOUT_BOTTOM:
+			case BRASERO_LAYOUT_RIGHT:
+				gtk_paned_pack1 (GTK_PANED (layout->priv->pane), source_pane, TRUE, TRUE);
+				gtk_paned_pack2 (GTK_PANED (layout->priv->pane), project_pane, TRUE, FALSE);
+				break;
+
+			default:
+				break;
+		}
+
+		g_object_unref (project_pane);
+		g_object_unref (source_pane);
+	}
+
+	if (layout->priv->preview_pane) {
+		brasero_layout_pack_preview (layout);
+		g_object_unref (layout->priv->preview_pane);
+	}
 
 	brasero_layout_size_reallocate (layout);
 }
 
+static void
+brasero_layout_HV_radio_button_toggled_cb (GtkRadioAction *radio,
+					   GtkRadioAction *current,
+					   BraseroLayout *layout);
+
+static void
+brasero_layout_type_changed_cb (GConfClient *client,
+				guint cxn,
+				GConfEntry *entry,
+				gpointer data)
+{
+	GSList *iter;
+	GSList *radios;
+	GtkAction *action;
+	GConfValue *value;
+	BraseroLayoutType layout_type;
+	BraseroLayout *layout = BRASERO_LAYOUT (data);
+
+	value = gconf_entry_get_value (entry);
+	if (value->type != GCONF_VALUE_INT)
+		return;
+
+	layout_type = gconf_value_get_int (value);
+	brasero_layout_change_type (layout, layout_type);
+
+	/* make sure our radio actions reflect the change */
+	action = gtk_action_group_get_action (layout->priv->action_group, "HView");
+
+	radios = gtk_radio_action_get_group (GTK_RADIO_ACTION (action));
+	for (iter = radios; iter; iter = iter->next)
+		g_signal_handlers_block_by_func (iter->data,
+						 brasero_layout_HV_radio_button_toggled_cb,
+						 layout);
+
+	gtk_radio_action_set_current_value (GTK_RADIO_ACTION (action),
+					    GTK_IS_VPANED (layout->priv->pane));
+
+	for (iter = radios; iter; iter = iter->next)
+		g_signal_handlers_unblock_by_func (iter->data,
+						   brasero_layout_HV_radio_button_toggled_cb,
+						   layout);
+
+	g_slist_free (radios);
+}
+
+static void
+brasero_layout_HV_radio_button_toggled_cb (GtkRadioAction *radio,
+					   GtkRadioAction *current,
+					   BraseroLayout *layout)
+{
+	guint layout_type;
+
+	if (gtk_radio_action_get_current_value (current))
+		layout_type = BRASERO_LAYOUT_BOTTOM;
+	else
+		layout_type = BRASERO_LAYOUT_RIGHT;
+
+	brasero_layout_change_type (layout, layout_type);
+
+	/* update the GConf key */
+	gconf_client_notify_remove (layout->priv->client,
+				    layout->priv->layout_notify);
+	layout->priv->layout_notify = 0;
+
+	gconf_client_set_int (layout->priv->client,
+			      BRASERO_KEY_DISPLAY_LAYOUT,
+			      layout_type,
+			      NULL);
+
+	/* This is to avoid a notification for change */
+	gconf_client_clear_cache (layout->priv->client);
+	layout->priv->layout_notify = gconf_client_notify_add (layout->priv->client,
+							       BRASERO_KEY_DISPLAY_LAYOUT,
+							       brasero_layout_type_changed_cb,
+							       layout,
+							       NULL,
+							       NULL);
+}
+
 static void
 brasero_layout_close_button_clicked_cb (GtkWidget *button,
 					BraseroLayout *layout)
@@ -1008,14 +1175,34 @@ brasero_layout_empty_toggled_cb (GtkToggleAction *action,
 				 BraseroLayout *layout)
 {
 	gboolean active;
+	GError *error = NULL;
 
 	active = gtk_toggle_action_get_active (action);
 	brasero_layout_set_side_pane_visible (layout, active);
 
-	if (!active)
-		brasero_layout_save (layout, BRASERO_LAYOUT_NONE_ID);
-	else if (layout->priv->active_item)
-		brasero_layout_save (layout, layout->priv->active_item->id);
+	if (layout->priv->sidepane_notify) {
+		gconf_client_notify_remove (layout->priv->client,
+					    layout->priv->sidepane_notify);
+		layout->priv->sidepane_notify = 0;
+	}
+
+	gconf_client_set_bool (layout->priv->client,
+			       BRASERO_KEY_SHOW_SIDEPANE,
+			       active,
+			       &error);
+
+	if (error) {
+		g_warning ("Can't set GConf key %s. \n", error->message);
+		g_error_free (error);
+		error = NULL;
+	}
+
+	layout->priv->sidepane_notify = gconf_client_notify_add (layout->priv->client,
+								 BRASERO_KEY_SHOW_SIDEPANE,
+								 brasero_layout_show_sidepane_changed_cb,
+								 layout,
+								 NULL,
+								 &error);
 }
 
 void
@@ -1115,6 +1302,16 @@ brasero_layout_class_init (BraseroLayoutClass *klass)
 	gtk_widget_class->show = brasero_layout_show;
 
 	gtk_object_class->destroy = brasero_layout_destroy;
+
+	brasero_layout_signals[SIDEPANE_SIGNAL] =
+	    g_signal_new ("sidepane", G_OBJECT_CLASS_TYPE (object_class),
+			  G_SIGNAL_ACTION | G_SIGNAL_RUN_FIRST,
+			  0,
+			  NULL, NULL,
+			  g_cclosure_marshal_VOID__BOOLEAN,
+			  G_TYPE_NONE,
+			  1,
+			  G_TYPE_BOOLEAN);
 }
 
 static void
@@ -1174,6 +1371,14 @@ brasero_layout_init (BraseroLayout *obj)
 	gtk_widget_show (obj->priv->pane);
 	gtk_box_pack_end (GTK_BOX (obj), obj->priv->pane, TRUE, TRUE, 0);
 
+	/* reflect that layout in the menus */
+	gtk_action_group_add_radio_actions (obj->priv->action_group,
+					    radio_entries,
+					    sizeof (radio_entries) / sizeof (GtkRadioActionEntry),
+					    GTK_IS_VPANED (obj->priv->pane),
+					    G_CALLBACK (brasero_layout_HV_radio_button_toggled_cb),
+					    obj);
+
 	/* remember the position */
 	position = gconf_client_get_int (obj->priv->client,
 					 BRASERO_KEY_DISPLAY_POSITION,
@@ -1204,14 +1409,21 @@ brasero_layout_init (BraseroLayout *obj)
 
 	if (obj->priv->layout_type == BRASERO_LAYOUT_TOP
 	||  obj->priv->layout_type == BRASERO_LAYOUT_LEFT)
-		gtk_paned_pack2 (GTK_PANED (obj->priv->pane), alignment, FALSE, FALSE);
+		gtk_paned_pack2 (GTK_PANED (obj->priv->pane), alignment, TRUE, TRUE);
 	else
-		gtk_paned_pack1 (GTK_PANED (obj->priv->pane), alignment, FALSE, FALSE);
+		gtk_paned_pack1 (GTK_PANED (obj->priv->pane), alignment, TRUE, TRUE);
 
 	obj->priv->main_box = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (alignment), obj->priv->main_box);
 	gtk_widget_show (obj->priv->main_box);
 
+	obj->priv->sidepane_notify = gconf_client_notify_add (obj->priv->client,
+							      BRASERO_KEY_SHOW_SIDEPANE,
+							      brasero_layout_show_sidepane_changed_cb,
+							      obj,
+							      NULL,
+							      NULL);
+
 	/* close button and  combo */
 	box = gtk_hbox_new (FALSE, 6);
 	gtk_widget_show (box);
